from django.contrib import admin
from django.urls import include, path

from allauth.account.decorators import secure_admin_login


admin.autodiscover()
admin.site.login = secure_admin_login(admin.site.login)


urlpatterns = [
    path("admin/", admin.site.urls),
    path("accounts/", include("allauth.urls")),
    path("_allauth/", include("allauth.headless.urls")),
    path("", include("allauth.idp.urls")),
    path("", include("tests.projects.common.urls")),
]
