# Generated by Django 4.2.16 on 2025-08-14 05:52

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('appauth', '0008_appusermodel_auth_config_userrolemodel_auth_config_and_more'),
        ('mfa', '0003_authenticator_type_uniq'),
    ]

    operations = [
        migrations.AlterField(
            model_name='authenticator',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='appauth.appusermodel'),
        ),
    ]
