# Generated by Django 4.2.16 on 2025-07-08 05:02

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('appauth', '0007_appuserauthtoken'),
        ('account', '0009_emailaddress_unique_primary_email'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailaddress',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='appauth.appusermodel', verbose_name='user'),
        ),
    ]
